"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const concatBuffers_1 = __importDefault(require("./concatBuffers"));
const encodeUtf8_1 = __importDefault(require("./encodeUtf8"));
const roundUp_1 = require("./roundUp");
const sum_1 = __importDefault(require("./sum"));
// Implementation derived from https://github.com/porsager/tarts
// and with help from https://en.wikipedia.org/wiki/Tar_(computing).
const headers = Object.entries({
    name: 100,
    mode: 8,
    uid: 8,
    gid: 8,
    size: 12,
    mtime: 12,
    chksum: 8,
    typeflag: 1,
    linkname: 100,
    magic: 6,
    version: 2,
    uname: 32,
    gname: 32,
    devmajor: 8,
    devminor: 8,
    prefix: 155,
});
const offsets = {};
headers.reduce((offset, [headerName, headerWidth]) => {
    offsets[headerName] = offset;
    return offset + headerWidth;
}, 0);
const BLKSIZE = 512;
exports.default = (files) => (0, concatBuffers_1.default)(files.map((f) => {
    // TODO Check file name length after encoding.
    const entry = {
        chksum: "        ",
        gid: f.gid ?? 0,
        gname: f.gname ?? "",
        magic: "ustar",
        mode: f.mode ?? 0o777,
        mtime: Math.floor((f.mtime ?? new Date()).getTime() / 1000),
        name: f.name,
        size: f.content.byteLength,
        typeflag: f.typeflag ?? "0",
        uid: f.uid ?? 0,
        uname: f.uname ?? "",
        version: "00",
    };
    const b = new Uint8Array(BLKSIZE + (0, roundUp_1.roundUp)(f.content.byteLength, BLKSIZE));
    const checksum = (0, sum_1.default)(headers.map(([headerName, headerWidth]) => {
        const entryVal = entry[headerName];
        if (entryVal === undefined) {
            return 0;
        }
        const value = (0, encodeUtf8_1.default)(typeof entryVal == "string"
            ? entryVal
            : entryVal.toString(8).padStart(headerWidth, "0"));
        b.set(value, offsets[headerName]);
        return (0, sum_1.default)(value);
    }));
    b.set((0, encodeUtf8_1.default)(`${checksum.toString(8).padStart(6, "0")}\0 `), offsets.chksum);
    b.set(f.content, BLKSIZE);
    return b;
}));
